var Bco54200_ExtratoBancario_Controller = function(ScreenService, General,restEngine, templateManager, Bco54200_ExtratoBancario_Service, ZHPromise, ZhFilterPreferences){
    /**************************************** Exception *****************************************************/

    var self = this;

    function exception(message) {
        this.message = message;
    }

    this.openFilter = function(){
        var extratoBancario = templateManager.container.getWidget('bco54200_extrato_bancario_grid');
        var data = moment().format('DD/MM/YYYY');
        var filterRow = {};
        filterRow.DTMOVREAL = {start : data, end : data};
        ZhFilterPreferences.setOriginalFilter(extratoBancario.id, filterRow);
        ZhFilterPreferences.openFilterWidget(extratoBancario, filterRow);
    };

    this.validateKeys = function(key, fieldName) {
        var integerNumbers = [48, 49, 50, 51,  52,  53,  54,  55,  56,  57];
        var validKeys = {};

        //build validation cases
        validKeys.integer = integerNumbers;
        validKeys.MSKINSJURFILI = integerNumbers.concat([45, 46, 47]); //'-', '.', '/'

        //check if there is any special validation
        if(fieldName && validKeys.hasOwnProperty(fieldName))
            return validKeys[fieldName].indexOf(key) < 0;
        else //returns generic one
            return validKeys.integer.indexOf(key) < 0;
    };

    /**************************************** Extrato Bancário *************************************************/

    /**
     * Essa função tem como finalidade fazer as validações necessárias  para a exclusão de
     * uma movimentação.
     *
     * @param  {Object} widget widget do extrato bancario (bco54200_extrato_bancario_grid)
     */
    this.delete = function(widget) {
        var currentRow = widget.currentRow;
        
        var param = [{
                'name': 'CDREDUCONT',
                'operator': '=',
                'value': currentRow.CDREDUCONT
            },
            {
                'name': 'CDSEQUEXTR',
                'operator': '=',
                'value': currentRow.CDSEQUEXTR
            },
            {
                'name': 'NRSEQMOVREAL',
                'operator': '=',
                'value': currentRow.NRSEQMOVREAL
            }
        ];
        
        restEngine.post('/buscaExtratoImportado', {
            "requestType": "Row",
            "row": param
        }).then((response) => {
            var retorno = response.dataset.conciliacao.conciliacao;
            if (retorno[0].IDSTCONMVBAN === '0') {
                if (retorno[0].IDTPMOVREAL != '1') {
                    ScreenService.confirmMessage(ScreenService.i18n('Atenção! Extrato bancário importado pelo sistema. Deseja excluir?'), 'question', function() {
                        widget.removeRow().then(function() {
                            ScreenService.closeSwipe().then(function() {
                                widget.activate();
                                widget.reload();
                                ScreenService.showMessage('Registro excluído com sucesso.');
                            });
                        }, function(error) {
                            ScreenService.showMessage(error.data.error);
                        });
                    });
                } else {
                    General.deleteCrud(widget).then(function() {
                        ScreenService.showMessage('Registro excluído com sucesso.');
                    });
                }
            } else {
                ScreenService.showMessage('Operação bloqueada! Extrato bancário está conciliado.');
            }
        });
    };

    /**
     * Essa função tem como finalidade fazer as validações necessárias para deixar, ou não,
     * uma movimentação ser editada.
     *
     * @param  {Object} widget widget do extrato bancario (bco54200_extrato_bancario_grid)
     */
    this.editar = function(widget) {
        var currentRow = widget.currentRow;
        if(currentRow.IDSTCONMVBAN != '0'){
            ScreenService.showMessage('Operação bloqueada! Extrato bancário está conciliado.');
        } else if(currentRow.IDTPMOVREAL != '1'){
            ScreenService.showMessage('Operação bloqueada! Extrato bancário foi importado.');
        } else {

            var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
            widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
            widget.edit();
        }
    };

    /**
     * Essa função tem como objetivo chamar as funções que geram o link para o relátorio,
     * abri-lo ou monstrar na tela o erro voltado pela requisição.
     * FUNÇÃO DESATUALIZADA. SUBSTITUIDO PELO DATAEXPORTER DO ZEEDHI
     */
    this.geraReport = function() {
        var widget = templateManager.container.getWidget('bco54200_extrato_bancario_grid');
        var widgetFilter = widget.appliedFilter;
        Bco54200_ExtratoBancario_Service.geraReport(widgetFilter.currentRow).then(function(birtURL){
            window.open(birtURL.messages[0].message);
        }, function(error) {
            if(error.data){
                ScreenService.showMessage(error.data.error);
            } else {
                ScreenService.showMessage(error.message);
            }
        });
    };

    /**
     * Essa função tem como finalidade adicinar em um field uma validação no seu keypress.
     * Essa validação adiciona so vai permitir que esse field rereba numeros inteiros.
     *
     * @param  {String} fieldName nome do filed que vai receber a regra
     */
    this.interger = function(fieldName) {
        var element = $("#" + fieldName);
        element.keypress(function(e) {
            if(this.validateKeys(e.which, fieldName)){
                e.preventDefault();
            }
        }.bind(this));
    };

    /************************************ Importação de Extrato Bancário ******************************************/

    /**
     * Essa função tem como objetivo verificar o valor do campo de banco da widget de
     * importação de extrato bancario. Quando o valor for alterado para null ou vazio
     * os dados das widgets de importação devem ser apenas zerados. Quando o campo tiver
     * o valor é preciso, além de zerar os valores anteriores da widget, chamar a rotina
     * de leitura de arquivo.
     *
     * @param  {Object} field field do banco
     */
    this.callFuncitionBancoField = function(field) {
        this.clearInfosImport();
        if(field.value() !== null || field.value() !== "") {
            this.uploadContentFile(field.widget);
        }
    };

    /**
     * Essa função tem como finalidade resetar o estado das widgets responsaveis pelo
     * processo de importação de extrato bancario, terem os valores dos campos, que
     * se referem as lidas no arquivo, resetados.
     */
    this.clearInfosImport = function() {
        var widgetExtrato = templateManager.container.getWidget('importacaoExtratoBancario');
        var widgetMovimentacoes = templateManager.container.getWidget('importacaoGrid');

        //Limpando campos de informações sobre o extrato a ser importado
        widgetExtrato.getField('CDCONTCORRMASK').setValue("");
        widgetExtrato.getField('DTSALDINIC').setValue("");
        widgetExtrato.getField('DTSALDFIN').setValue("");
        widgetExtrato.getField('SALDOINICIAL').setValue("");
        widgetExtrato.getField('SALDOFINAL').setValue("");

        //Limpando as movimentações do grid
        widgetMovimentacoes.dataSource.data = [];
        widgetMovimentacoes.redraw();
        
    };

    /**
     * Essa função tem como finalidade retornar todas as widgets envolvidas no processo de
     * importação de extrato bancario para a configurar default da widget. Campo de banco
     * desabilitado para edição e sem requerimento; todos os campos limpos; data do grid
     * zerado.
     */
    this.defaultStatusWidget = function() {
        var widget = templateManager.container.getWidget('importacaoExtratoBancario');
        widget.getField('NMBANCO').readOnly = true;
        delete widget.getField('NMBANCO').validations.required;
        widget.currentRow.NMBANCO = null;
        widget.currentRow.CDBANCO = null;
        this.clearInfosImport();
    };

    /**
     * Essa função tem como finalidade deletar uma movimentação (no grid de importação), quando não é mais necessária que
     * aquela movimentação, do extrato importado, não seja salva. Paro isso, as linhas selecionadas são excluidas do data
     * do dataSource da widget (importacaoGrid), e o novo valor final é calculado e colocado na currentRow da widget
     * importacaoExtratoBancario.
     *
     * @param  {Object} widget widget importacaoGrid, que contém as movimentações do extrato que será importado
     */
    this.deletMovImport = function(widget) {
        var checkedRows = widget.dataSource.checkedRows;
        var data = widget.dataSource.data;
        var saldoini = widget.getParent().currentRow.SALDOINICIAL;
        data = data.filter(function(movimentacao){
            return checkedRows.indexOf(movimentacao) === -1;
        });
        var newVrfin = this.recalcVrfinal(data, saldoini);
        widget.dataSource.data = data;
        widget.getParent().currentRow.SALDOFINAL = newVrfin;
    };

    /**
     * Está função é chamado no evento da action (importar) da widget extrato bancário (bco54200_extrato_bancario_grid).
     * Ela torna a widget para importação de extrato (importacaoExtratoBancario) bancário visível e a coloca no modo de
     * edição. Além disso usa tal widget para abrir um swipe, onde sera feita a importação do extrato bancário.
     *
     * @param  {Object} widget widget que sera usada para abrir o swipe (importacaoExtratoBancario)
     */
    this.openSwipeForImport = function() {
        var widgetImportacao = templateManager.container.getWidget('importacaoExtratoBancario');
        var widgetGrid = templateManager.container.getWidget('importacaoGrid');
        
        widgetImportacao.currentRow = {};
        widgetImportacao.currentRow.__is_new = true;
        
        ScreenService.openSwipe(widgetImportacao).then(function(){
            self.viewMode(widgetGrid);
			
			widgetImportacao.templateToForm = "widget/tabbedWidget.html";
			widgetImportacao.templateToView = "widget/tabbedWidget.html";
			widgetImportacao.formTemplate = "widget/tabbedWidget.html";
			
			widgetGrid.dataSource.data = [];
			setTimeout(function(){ 
			
    			widgetGrid.list();
    			widgetImportacao.childTemplate = "widget/form.html";
    			templateManager.updateTemplate(); widgetGrid.activate();
    			
    			$($('section.zh-widget.ng-scope')[1]).children().css('top', (parseInt($('section.zh-field-group.ng-scope').css('height').replace('px', '')) + 40 ) + 'px');
    			
    			$(window).resize();
			
			}, 1000);
		});
    };
    
    this.viewMode = function (widget) {
        widget.view();
        templateManager.updateTemplate();
    };
    
    /* Função antiga para abrir a importação
    
    this.openSwipeForImport = function() {
        var widgetImport = templateManager.container.getWidget('importacaoExtratoBancario');
        widgetImport.isVisible = true;
        ScreenService.openSwipe(widgetImport).then(function(){
            widgetImport.editing = true;
            //widgetImport.widgets[0].list();
        }.bind(this));
    };*/

    /**defaultStatusWidget
     * Essa função é chamada no evneto da action (cancelar) da widget grid de importação da extrato bancário (importacaoGrid).
     * Ela fecha todos os swipes abertos e torna a widget (bco54200_extrato_bancario_grid) de extrato bancário ativa.
     *
     * @param  {String} widget widget pai da widget importacao de extrato bancário (bco54200_extrato_bancario_grid)
     */
    this.backToExtrato = function() {
        var tabbed = templateManager.container.getWidget('importacaoExtratoBancario');
        ScreenService.goBack();
        ScreenService.closeSwipe().then(function(){
            tabbed.isVisible = false;
            this.defaultStatusWidget();
            //tabbed.list();
            //tabbed.widgets[0].list();
            tabbed.getParent().activate();
        }.bind(this));
    };

    /**
     * Essa função te como objetivo, receber um data de um widget e recalcular o saldo final do extrato. Para isso é calcular
     * a soma dos valores de cada movimentação passa e subitrair do saldo inicial do extrato
     *
     * @param  {Array} data  objetos com os dados para semre recalculados
     * @param  {String} saldo valor com o saldo inicial do extrato
     */
    this.recalcVrfinal = function(data, saldo) {
        var saldoMovimentacao = 0;
        data.forEach(function(movimentacao) {
            if (movimentacao.IDMOVREAL === 'D') { // D = debito, é preciso passar o valor para < 0
                saldoMovimentacao += (movimentacao.VRMOVREAL * (-1));
            } else if (movimentacao.IDMOVREAL === 'C'){ // C = crédito, é preciso apenas somar
                saldoMovimentacao += movimentacao.VRMOVREAL;
            }
        });
        var newSaldo = saldo + saldoMovimentacao;
        return newSaldo;
    };

    this.setCdreducont = function(extrato, conta, widget, tipo) {
        var movis = extrato.movreal;
        movis.forEach(function(movi){
            movi.CDREDUCONT = movi.CDREDUCONT[conta];
        });
        this.setDataOfGrid(widget, extrato, tipo);
    };

    /**
     * Essa função manda todos o dados do data do dataSource da widget de grid de importação de extrato bancario, tratando
     * todos os erros.
     *
     * @param  {Object} widget widget grid de importação de extrat bancário
     */
    this.saveImport = function(widget) {
        if(widget.getParent().getField('UPLOADFILE').value()) {
            var data = widget.dataSource.data;
            if (data.length !== 0) {
                Bco54200_ExtratoBancario_Service.saveImport(data).then(
                    function(response){
                        //Limpa os dados da widget, para possibilitar uma nova importação
                        this.defaultStatusWidget();
                        widget.getParent().getField('UPLOADFILE').fileList[0].name = '';
                        ScreenService.closeSwipe().then(function(){
                            //Recarrega a widget de movimentações
                            widget.getParent().getParent().reload().then(function(){
                                if (response.dataset.MOVDUPLICADOS.length > 0){
                                    var movimentosDuplicados = templateManager.container.getWidget('movimentosDuplicados');
                                    movimentosDuplicados.dataSource.data = response.dataset.MOVDUPLICADOS;
                                    ScreenService.openPopup(movimentosDuplicados).then(function(){
                                        templateManager.container.getWidget('bco54200_extrato_bancario_grid').reload();
                                        ScreenService.showMessage('Os seguintes movimentos não foram importados por estarem em duplicidade no Arquivo (Valor, Data, Documento).');
                                    });
                                }else{
                                    ScreenService.showMessage('Importação realizada com sucesso.');
                                }
                            });
                            widget.getParent().getParent().activate();
                            widget.getParent().isVisible = false;
                        });

                    }.bind(this),
                    function(error){
                        ScreenService.showMessage(error.data.error);
                    }
                );
            } else {
                ScreenService.showMessage('Operação bloqueada. Não há dado(s) a ser(em) importado(s).');
            }
        } else {
            ScreenService.showMessage('Operação bloqueada. Informe o arquivo para importação.');
        }
    };

    /**
     * Essa fução tem como finalidade pegar o resultado do dataset que ela receber e satar o data do dataSource da widget
     * filha da importacaoExtratoBancario, e fazer os dados, lidos pelo backend serem mostrados no grid
     *
     * @param {Oject} widget   widget filha da widget de iporatação e de extrato bancario
     * @param {Array} dataSets dataset retornado da leitura do arquivo pelo backned
     */
    this.setDataOfGrid = function(widget, dataSets, tipo) {
        widget.widgets[0].dataSource.data = dataSets.movreal;
        widget.widgets[0].redraw();

        widget.getField('CDCONTCORRMASK').setValue(dataSets.extrbanco.CDCONTCORRMASK);
        widget.getField('DTSALDINIC').setValue(dataSets.extrbanco.DTSALDINIC);
        widget.getField('DTSALDFIN').setValue(dataSets.extrbanco.DTSALDFIN);
        widget.getField('SALDOINICIAL').setValue(dataSets.extrbanco.SALDOINICIAL);
        widget.getField('SALDOFINAL').setValue(dataSets.extrbanco.SALDOFINAL);

        if (tipo === "OFX") {
            widget.getField('NMBANCO').setValue(dataSets.extrbanco.NMBANCO);
        }
    };

    /**COMPLETAR DOCUMENTAÇÃO
     * Essa função tem como finalidade enviar ao seviço o conteudo do arquivo (convertido em base 64) e, se a extenção
     * do arquivo for TXT, informar de qual banco é o arquivo.
     *
     * @param  {Object} widget recebe a widget de importacao de e xtrato bancário (importacaoExtratoBancario)
     */
    this.uploadContentFile = function(widget) {
        try {
            if(widget.isValid()){
                var valueField = widget.currentRow.UPLOADFILE[0];
                var banco = "";
                var agencia = "";
                var tipo = "OFX";
                if(valueField.type === 'text/plain'){
                    agencia = widget.currentRow.CDAGENCIA;
                    banco = widget.currentRow.CDBANCCNAB;
                    tipo = "TXT";
                }
                Bco54200_ExtratoBancario_Service.uploadContentFile(valueField.b64File, banco, tipo).then(function(callback){
                    var extrato = callback.dataset;
                    var contas = extrato.extrbanco.CONTAS;
                    if (contas > 1) {
                        this.validaConta(extrato, widget, tipo);
                    } else {
                        this.setDataOfGrid(widget, extrato, tipo);
                    }
                }.bind(this), function(error){
                    ScreenService.showMessage(error.data.error);
                });
            }
        } catch (e) {
            ScreenService.showMessage(e.message);
        }
    };

    this.validaConta = function(extrato, widget, tipo) {
        var tpContas = extrato.extrbanco.TPCONTAS;
        var actions = [];
        var self = this;
        tpContas.forEach(function(conta){
            var optionLabel = conta === 'C' ? 'Circulante' : conta === 'A' ? 'Aplicação' : 'Garantida';
            var option = {
                label: ScreenService.i18n(optionLabel),
                "default": true,
                code: function(e, deferred) {
                    self.setCdreducont(extrato, conta, widget, tipo);
                }
            };
            actions.push(option);
        });
        ScreenService.showCustomDialog('Importar extrato bancário para a conta do tipo ?', 'CONFIRMATION', 'question', actions);
    };

    /**
     * Essa função recebe tem a finalidade de verificar qual a extenssão do arquivo que o usuário selecionou, e quando
     * do tipo TXT, habilidar o campo de banco para que o usuário informe o banco, dessa forma o backend sabera qual o
     * layout que devera ser utilizado para a leitura do arquivo. Quando OFX o layout utiliado é padrão para todos os
     * bancos
     *
     * @param  {[type]} widget recebe a widget de importacao de e xtrato bancário (importacaoExtratoBancario)
     */
    this.validTypeFile = function(widget) {
        var valueField = widget.currentRow.UPLOADFILE[0];
        var file = valueField.b64File.split(',');
        if(file[1]) {
            var fieldBanco = widget.getField('NMBANCO');
            if(valueField.type === 'text/plain'){
                fieldBanco.readOnly = false;
                fieldBanco.validations.required = true;
                ScreenService.showMessage('Informe o banco referente ao extrato selecionado.');
            } else {
                fieldBanco.readOnly = true;
                delete fieldBanco.validations.required;
                this.uploadContentFile(widget);
            }
        } else {
            valueField.setValue(null);
            widget.currentRow.valueField = null;
            ScreenService.showMessage('Arquivo vazio.');
        }
    };
    
    
    /**
     * Novo Extrato Bancário
     */
    this.newExtrato = function() {
        var widget = templateManager.container.getWidget('bco54200_extrato_bancario_grid');
        var filtro = widget.appliedFilter;
        
        widget.newRow();
        
        if(filtro){
            widget.currentRow.CDREDUCONT = filtro.FILTER_ROW.CDREDUCONT;
            widget.currentRow.CDBANCO    = filtro.FILTER_ROW.CDBANCO;
            widget.currentRow.CDAGENCIA  = filtro.FILTER_ROW.CDAGENCIA;
            widget.currentRow.CDCONTCORR = filtro.FILTER_ROW.CDCONTCORR;
            widget.currentRow.DSCONTCORR = filtro.FILTER_ROW.DSCONTCORR;
        }
    };

};
Configuration(function (ContextRegister){
    ContextRegister.register('Bco54200_ExtratoBancario_Controller', Bco54200_ExtratoBancario_Controller);
});
